@@ -1,6 +1,6 @@
 #!/usr/local/bin/perl
 # $File: //member/autrijus/PAR/script/par.pl $ $Author: autrijus $
-# $Revision: #88 $ $Change: 9561 $ $DateTime: 2004/01/03 04:22:17 $ vim: expandtab shiftwidth=4
+# $Revision: #98 $ $Change: 9867 $ $DateTime: 2004/02/02 23:24:48 $ vim: expandtab shiftwidth=4
 
 package __par_pl;
 
@@ -32,18 +32,18 @@ To manipulate a I<PAR distribution>:
 
 To use F<Hello.pm> from F<./foo.par>:
 
-    % par.pl -A./foo.par -MHello 
+    % par.pl -A./foo.par -MHello
     % par.pl -A./foo -MHello    # the .par part is optional
 
 Same thing, but search F<foo.par> in the F<@INC>;
 
-    % par.pl -Ifoo.par -MHello 
+    % par.pl -Ifoo.par -MHello
     % par.pl -Ifoo -MHello      # ditto
 
 Run F<test.pl> or F<script/test.pl> from F<foo.par>:
 
     % par.pl foo.par test.pl    # looks for 'main.pl' by default,
-                                # otherwise run 'test.pl' 
+                                # otherwise run 'test.pl'
 
 To make a self-containing script containing a PAR file :
 
@@ -97,7 +97,7 @@ PAR file:
     % ./myapp                   # run it anywhere without perl binaries
 
 With the C<--par-options> flag, generated binaries can act as C<parl>
-to pack new binaries: 
+to pack new binaries:
 
     % ./myapp --par-options -Omyap2 myapp.par   # identical to ./myapp
     % ./myapp --par-options -Omyap3 myap3.par   # now with different PAR
@@ -150,8 +150,13 @@ followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
 
 =cut
 
-my @tmpfiles;
-END { if (@tmpfiles) { unlink @tmpfiles; rmdir $par_temp } }
+my ($par_temp, $progname, @tmpfile);
+END { if ($ENV{PAR_CLEAN}) {
+    unlink @tmpfile;
+    rmdir $par_temp;
+    $par_temp =~ s{[^\\/]*[\\/]?$}{};
+    rmdir $par_temp;
+} }
 
 BEGIN {
     Internals::PAR::BOOT() if defined &Internals::PAR::BOOT;
@@ -163,7 +168,6 @@ if ($ENV{PAR_ARGV_0}) {
     $0 = $ENV{PAR_ARGV_0};
 }
 
-my $progname = $0;
 my $quiet = !$ENV{PAR_DEBUG};
 
 # fix $progname if invoked from PATH
@@ -173,21 +177,15 @@ my %Config = (
     _delim      => ($^O =~ /^MSWin|OS2/ ? '\\' : '/'),
 );
 
-if (-s "$progname$Config{_exe}") {
-    $progname = "$progname$Config{_exe}";
-}
-elsif (!-s $progname) {
-    foreach my $dir (split /\Q$Config{path_sep}\E/, $ENV{PATH}) {
-        $dir =~ s/\Q$Config{_delim}\E$//;
-        (($progname = "$dir$Config{_delim}$progname$Config{_exe}"), last)
-            if -s "$dir$Config{_delim}$progname$Config{_exe}";
-        (($progname = "$dir$Config{_delim}$progname"), last)
-            if -s "$dir$Config{_delim}$progname";
-    }
-}
-
+_par_init_env();
+_set_progname();
 _set_par_temp();
 
+# Append ".exe" to $0 if it was missing
+$0 .= $Config{_exe} if $Config{_exe}
+    and $progname =~ /\Q$Config{_exe}\E$/
+    and $0 !~ /\Q$Config{_exe}\E$/;
+
 # Magic string checking and extracting bundled modules {{{
 my ($start_pos, $data_pos);
 {
@@ -232,8 +230,6 @@ my ($start_pos, $data_pos);
                 close $out;
                 chmod 0755, $filename;
             }
-            $PAR::Heavy::DLCache{$filename}++;
-            $PAR::Heavy::DLCache{$basename}   =
             $PAR::Heavy::FullCache{$fullname} = $filename;
             $PAR::Heavy::FullCache{$filename} = $fullname;
         }
@@ -251,6 +247,7 @@ my ($start_pos, $data_pos);
             $PAR::Heavy::ModuleCache{$fullname} = {
                 buf => $buf,
                 crc => $crc,
+                name => $fullname,
             };
         }
         read _FH, $buf, 4;
@@ -273,7 +270,7 @@ my ($start_pos, $data_pos);
 
         $INC{$module} = "/loader/$filename/$module";
 
-        if ($ENV{PAR_CLEARTEMP} and defined(&IO::File::new)) {
+        if ($ENV{PAR_CLEAN} and defined(&IO::File::new)) {
             my $fh = IO::File->new_tmpfile or die $!;
             binmode($fh);
             print $fh $filename->{buf};
@@ -380,7 +377,18 @@ if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
 
 # Output mode (-O) handling {{{
 if ($out) {
+    require IO::File;
+    require Archive::Zip;
+
     my $par = shift(@ARGV);
+    my $zip = Archive::Zip->new($par) if defined($par);
+    my %env = do {
+        if ($zip and my $meta = $zip->contents('META.yml')) {
+            $meta =~ s/.*^par:$//ms;
+            $meta =~ s/^\S.*//ms;
+            $meta =~ /^  ([^:]+): (.+)$/mg;
+        }
+    };
 
     # Open input and output files {{{
     local $/ = \4;
@@ -391,8 +399,6 @@ if ($out) {
         die "$par is not a PAR file" unless <PAR> eq "PK\003\004";
     }
 
-    require IO::File;
-    require Archive::Zip;
     my $fh = IO::File->new(
 	$out,
 	IO::File::O_CREAT() | IO::File::O_WRONLY() | IO::File::O_TRUNC(),
@@ -407,6 +413,15 @@ if ($out) {
         require PAR::Filter::PodStrip;
         PAR::Filter::PodStrip->new->apply(\$loader, $0)
     }
+    foreach my $key (sort keys %env) {
+        my $val = $env{$key} or next;
+        $val = eval $val if $val =~ /^['"]/;
+        my $magic = "__ENV_PAR_" . uc($key) . "__";
+        my $set = "PAR_" . uc($key) . "=$val";
+        $loader =~ s{$magic( +)}{
+            $magic . $set . (' ' x (length($1) - length($set)))
+        }eg;
+    }
     $fh->print($loader);
     $/ = undef;
     # }}}
@@ -423,7 +438,7 @@ if ($out) {
         } grep {
             !/BSDPAN/
         } grep {
-            ($bundle ne 'site') or 
+            ($bundle ne 'site') or
             ($_ ne $Config::Config{archlibexp} and
              $_ ne $Config::Config{privlibexp});
         } @INC;
@@ -463,7 +478,10 @@ if ($out) {
 
             next unless defined $name and not $written{$name}++;
             next if !ref($file) and $file =~ /\.\Q$lib_ext\E$/;
-            outs(qq(Packing "$file"...));
+            outs( join "",
+                qq(Packing "), ref $file ? $file->{name} : $file,
+                qq("...)
+            );
 
             my $content;
             if (ref($file)) {
@@ -492,10 +510,7 @@ if ($out) {
     # }}}
 
     # Now write out the PAR and magic strings {{{
-    if (defined($par)) {
-        my $zip = Archive::Zip->new($par);
-        $zip->writeToFileHandle($fh);
-    }
+    $zip->writeToFileHandle($fh) if $zip;
 
     $fh->print(pack('N', $fh->tell - length($loader)));
     $fh->print("\nPAR.pm\n");
@@ -601,11 +616,21 @@ sub _set_par_temp {
 
         my $stmpdir = "$path$Config{_delim}par-$username";
         mkdir $stmpdir, 0755;
-        if (!$ENV{PAR_CLEARTEMP} and my $mtime = (stat($progname))[9]) {
-            $stmpdir .= "$Config{_delim}cache-$mtime";
+        if (!$ENV{PAR_CLEAN} and my $mtime = (stat($progname))[9]) {
+            my $ctx = eval { require Digest::SHA1; Digest::SHA1->new }
+                   || eval { require Digest::MD5; Digest::MD5->new };
+                
+            if ($ctx and open(my $fh, "<$progname")) {
+                binmode($fh);
+                $ctx->addfile($fh);
+                close($fh);
+            }
+
+            $stmpdir .= "$Config{_delim}cache-"
+                     . ( $ctx ? $ctx->hexdigest : $mtime );
         }
         else {
-            $ENV{PAR_CLEARTEMP} = 1;
+            $ENV{PAR_CLEAN} = 1;
             $stmpdir .= "$Config{_delim}temp-$$";
         }
 
@@ -620,12 +645,12 @@ sub _set_par_temp {
 sub _tempfile {
     my ($ext, $crc) = @_;
     my ($fh, $filename);
-    
+
     $filename = "$par_temp/$crc$ext";
 
-    if ($ENV{PAR_CLEARTEMP}) {
+    if ($ENV{PAR_CLEAN}) {
         unlink $filename if -e $filename;
-        push @tmpfiles, $filename;
+        push @tmpfile, $filename;
     }
     else {
         return (undef, $filename) if (-r $filename);
@@ -636,6 +661,43 @@ sub _tempfile {
     return($fh, $filename);
 }
 
+sub _set_progname {
+    if ($ENV{PAR_PROGNAME} and $ENV{PAR_PROGNAME} =~ /(.+)/) {
+        $progname = $1;
+    }
+    return if -s ($progname ||= $0);
+
+    if (-s "$progname$Config{_exe}") {
+        $ENV{PAR_PROGNAME} = $progname = "$progname$Config{_exe}";
+        $0 .= $Config{_exe};
+        return $progname;
+    }
+
+    foreach my $dir (split /\Q$Config{path_sep}\E/, $ENV{PATH}) {
+        $dir =~ s/\Q$Config{_delim}\E$//;
+        (($progname = "$dir$Config{_delim}$progname$Config{_exe}"), last)
+            if -s "$dir$Config{_delim}$progname$Config{_exe}";
+        (($progname = "$dir$Config{_delim}$progname"), last)
+            if -s "$dir$Config{_delim}$progname";
+    }
+
+    $ENV{PAR_PROGNAME} = $progname;
+}
+
+sub _par_init_env {
+    return if $ENV{PAR_INITIALIZED}++;
+
+    my $par_clean = "__ENV_PAR_CLEAN__               ";
+
+    if ($ENV{PAR_TEMP}) {
+        delete $ENV{PAR_CLEAN};
+    }
+    elsif (!$ENV{PAR_CLEAN}) {
+        my $value = substr($par_clean, 12 + length("CLEAN"));
+        $ENV{PAR_CLEAN} = $1 if $value =~ /^PAR_CLEAN=(\S+)/;
+    }
+}
+
 sub outs { warn("@_\n") unless $quiet }
 
 sub init_inc {
